"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeMasmottTriggers = exports.createViewDoc = void 0;
const lodash_1 = require("lodash");
const firebase_admin_1 = require("./firebase-admin");
const firebase_functions_1 = require("./firebase-functions");
const util_1 = require("./util");
const view_count_1 = require("./view-count");
const view_join_1 = require("./view-join");
/**
 * Materialize view document data based on given specification.
 *
 * @param app Firebase app.
 * @param srcDocData Data of the view's source document.
 * @param viewSpec Specification of the view.
 * @returns Materialized view data.
 */
async function materializeViewData(app, srcDocData, { selectedFieldNames, joinSpecs, countSpecs }) {
    const selectViewData = (0, lodash_1.pick)(srcDocData, selectedFieldNames);
    const joinViewData = await (0, view_join_1.materializeJoinViewData)(app, srcDocData, joinSpecs);
    const countViewData = (0, view_count_1.materializeCountViewData)(countSpecs);
    const materializedViewData = Object.assign(Object.assign(Object.assign({}, selectViewData), joinViewData), countViewData);
    return materializedViewData;
}
/**
 * Materialize and create a view document based on given specification.
 *
 * @param app Firebase app.
 * @param collectionName Name of the view's collection.
 * @param viewName Name of the view.
 * @param srcDoc Document snapshot of the view's source document
 * @param viewSpec Specification of the view.
 * @returns Creation result of the view document.
 */
async function createViewDoc(app, collectionName, viewName, srcDoc, viewSpec) {
    const viewDocData = await materializeViewData(app, srcDoc.data, viewSpec);
    const viewDocId = srcDoc.id;
    const viewCollectionName = (0, util_1.getViewCollectionName)(collectionName, viewName);
    return (0, firebase_admin_1.createDoc)(app, viewCollectionName, viewDocId, viewDocData);
}
exports.createViewDoc = createViewDoc;
/**
 * Make a trigger to run on source document creation. The trigger will
 * materialize and create view documents with the same id as updated source
 * document, with materialized view data.
 *
 * @param app Firebase app.
 * @param collectionName Name of the view's collection
 * @param viewName Name of the view.
 * @param viewSpec Specification of the
 * @returns Trigger on view's source document created.
 */
function onSrcDocCreated(app, collectionName, viewName, viewSpec) {
    return (0, firebase_functions_1.makeOnCreateTrigger)(collectionName, async (srcDoc) => createViewDoc(app, collectionName, viewName, srcDoc, viewSpec));
}
/**
 * Make a trigger to run on source document update. The trigger will update all
 * view documents with the same id as updated source document, if there is a
 * change.
 *
 * @param app Firebase app.
 * @param collectionName
 * @param viewName
 * @param selectedFieldNames
 * @returns
 */
function onSrcDocUpdated(app, collectionName, viewName, selectedFieldNames) {
    return (0, firebase_functions_1.makeOnUpdateTrigger)(collectionName, async (srcDoc) => {
        const allDocDataUpdate = (0, util_1.getDocDataChange)(srcDoc.data);
        const selectViewUpdateData = (0, lodash_1.pick)(allDocDataUpdate, selectedFieldNames);
        // Changing referenceId is not supported at the moment.
        const joinViewUpdateData = {};
        const viewUpdateData = Object.assign(Object.assign({}, selectViewUpdateData), joinViewUpdateData);
        if ((0, lodash_1.isEmpty)(viewUpdateData)) {
            return;
        }
        const viewDocId = srcDoc.id;
        const viewCollectionName = (0, util_1.getViewCollectionName)(collectionName, viewName);
        await (0, firebase_admin_1.updateDoc)(app, viewCollectionName, viewDocId, viewUpdateData);
    });
}
/**
 * Make a trigger to run on source document delete. The trigger will delete all
 * view documents with the same id as deleted source document.
 *
 * @param app Firebase app.
 * @param collectionName
 * @param viewName
 * @returns
 */
function onSrcDocDeleted(app, collectionName, viewName) {
    return (0, firebase_functions_1.makeOnDeleteTrigger)(collectionName, async (srcDoc) => {
        const viewDocId = srcDoc.id;
        const viewCollectionName = (0, util_1.getViewCollectionName)(collectionName, viewName);
        await (0, firebase_admin_1.deleteDoc)(app, viewCollectionName, viewDocId);
    });
}
/**
 * Make a trigger to run on deletion of a document referenced by source
 * document. The trigger will delete all document that refers to that referenced
 * document.
 *
 * @param app Firebase app.
 * @param collectionName
 * @param src
 * @returns
 */
function onSrcRefDocDeleted(app, collectionName, src) {
    return (0, lodash_1.mapValues)(src, (sourceField, sourceFieldName) => {
        // only create trigger if the field is type of refId (the document has
        // reference to another document)
        if (sourceField.type !== 'refId') {
            return undefined;
        }
        const { refCollection } = sourceField;
        const refidFieldName = sourceFieldName;
        return (0, firebase_functions_1.makeOnDeleteTrigger)(refCollection, async (refDoc) => {
            const referrerSrcDocsSnapshot = await (0, firebase_admin_1.getCollection)(app, collectionName, (collection) => collection.where(refidFieldName, '==', refDoc.id));
            const referrerDocsDeletes = referrerSrcDocsSnapshot.docs.map((doc) => (0, firebase_admin_1.deleteDoc)(app, collectionName, doc.id));
            await Promise.allSettled(referrerDocsDeletes);
        });
    });
}
/**
 * Make triggers for a view.
 *
 * @param app Firebase app.
 * @param collectionName Name of the view's collection.
 * @param viewName Name of the view.
 * @param viewSpec Specification of the view.
 * @returns
 */
function makeViewTriggers(app, collectionName, viewName, viewSpec) {
    return {
        onSrcDocCreated: onSrcDocCreated(app, collectionName, viewName, viewSpec),
        onSrcDocUpdated: onSrcDocUpdated(app, collectionName, viewName, viewSpec.selectedFieldNames),
        onSrcDocDeleted: onSrcDocDeleted(app, collectionName, viewName),
        onJoinRefDocUpdated: (0, view_join_1.onJoinRefDocUpdated)(app, collectionName, viewName, viewSpec.joinSpecs),
        onCountedDocCreated: (0, view_count_1.onCountedDocCreated)(app, collectionName, viewName, viewSpec.countSpecs),
        onCountedDocDeleted: (0, view_count_1.onCountedDocDeleted)(app, collectionName, viewName, viewSpec.countSpecs),
    };
}
/**
 * Make triggers for a collection.
 *
 * @param app Firebase app.
 * @param collectionSpec Specification of the collection.
 * @param collectionName Name of the collection.
 * @returns Triggers made for the collection.
 */
function makeCollectionTriggers(app, { src, views }, collectionName) {
    return {
        onRefDocDeleted: onSrcRefDocDeleted(app, collectionName, src),
        view: (0, lodash_1.mapValues)(views, (view, viewName) => makeViewTriggers(app, collectionName, viewName, view)),
    };
}
/**
 * Make triggers for masmott.
 *
 * @param app Firebase app.
 * @param collectionSpecs Collections specs of the app.
 * @returns Triggers made by masmott.
 */
function makeMasmottTriggers(app, collectionSpecs) {
    return (0, lodash_1.mapValues)(collectionSpecs, (collectionSpec, collectionName) => makeCollectionTriggers(app, collectionSpec, collectionName));
}
exports.makeMasmottTriggers = makeMasmottTriggers;
//# sourceMappingURL=get-trigger.js.map