"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapFirebaseChangeSnapshot = exports.wrapFirebaseSnapshot = exports.getViewCollectionName = exports.getDocDataChange = exports.getValueChange = exports.compactObject = exports.mergeObjectArray = exports.throwRejectedPromises = exports.mapValues = void 0;
const lodash_1 = require("lodash");
function mapValues(obj, mapper) {
    return (0, lodash_1.mapValues)(obj, mapper);
}
exports.mapValues = mapValues;
/**
 * Throw rejected promises from array of settled promises.
 *
 * @param promiseResults The array of settled promises to process
 */
function throwRejectedPromises(promiseResults
// eslint-disable-next-line functional/no-return-void
) {
    const rejectedPromiseReasons = promiseResults.map((result) => result.status === 'rejected' ? result.reason : undefined);
    const errors = (0, lodash_1.compact)(rejectedPromiseReasons);
    if (!(0, lodash_1.isEmpty)(errors)) {
        throw Error(JSON.stringify(errors));
    }
}
exports.throwRejectedPromises = throwRejectedPromises;
/**
 * Create a object by merging array of objects.
 *
 * @param objectArray The array of object to merge.
 * @returns The merged object
 */
function mergeObjectArray(objectArray) {
    return objectArray.reduce((acc, object) => (Object.assign(Object.assign({}, acc), object)), {});
}
exports.mergeObjectArray = mergeObjectArray;
/**
 * Create an object with all undefined values removed.
 *
 * @example
 * const obejectWithUndefined = {
 *   definedValue: '',
 *   undefinedValue: undefined
 *  }
 * const result = compactObject(objectWithUndefined)
 * // result => { definedValue: '' }
 *
 *
 * @param object The object to compact.
 * @returns Returns the new object without undefined values.
 */
function compactObject(object) {
    return Object.entries(object).reduce((acc, [key, value]) => {
        if (value !== undefined) {
            return Object.assign(Object.assign({}, acc), { [key]: value });
        }
        return acc;
    }, {});
}
exports.compactObject = compactObject;
/**
 * Get value difference before and after a change.
 * Returns undefined if value not changed.
 *
 * @example
 * const change1 = getValueChange('foo', 'bar')
 * // change1 => 'bar'
 *
 * @example
 * const change2 = getValueChange('lorem', 'lorem')
 * // change2 => undefined
 *
 * @param beforeValue Value before change.
 * @param afterValue Value after change.
 * @returns Difference between before and after.
 */
function getValueChange(beforeValue, afterValue) {
    if (typeof beforeValue === 'string' && typeof afterValue === 'string') {
        if (beforeValue !== afterValue) {
            return afterValue;
        }
        return undefined;
    }
    throw Error(JSON.stringify({ beforeValue, afterValue }));
}
exports.getValueChange = getValueChange;
/**
 *
 * @param documentDataChange
 * @returns
 */
function getDocDataChange({ before, after, }) {
    const docDataDiff = mapValues(before, (beforeFieldData, fieldName) => {
        const afterFieldData = after[fieldName];
        // undefined (optional) field is not supported
        if (afterFieldData === undefined) {
            throw Error(JSON.stringify({ before, afterFieldData, fieldName }));
        }
        const fieldDiff = getValueChange(beforeFieldData, afterFieldData);
        return fieldDiff;
    });
    const compactDocDataDiff = compactObject(docDataDiff);
    return compactDocDataDiff;
}
exports.getDocDataChange = getDocDataChange;
function getViewCollectionName(collectionName, viewName) {
    return `${collectionName}_${viewName}`;
}
exports.getViewCollectionName = getViewCollectionName;
function wrapFirebaseSnapshot(snapshot) {
    var _a;
    return {
        id: snapshot.id,
        data: (_a = snapshot.data()) !== null && _a !== void 0 ? _a : {},
    };
}
exports.wrapFirebaseSnapshot = wrapFirebaseSnapshot;
function wrapFirebaseChangeSnapshot(change) {
    return {
        id: change.after.id,
        data: {
            before: change.before.data(),
            after: change.after.data(),
        },
    };
}
exports.wrapFirebaseChangeSnapshot = wrapFirebaseChangeSnapshot;
//# sourceMappingURL=util.js.map