"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onJoinRefDocUpdated = exports.materializeJoinViewData = void 0;
const lodash_1 = require("lodash");
const firebase_admin_1 = require("./firebase-admin");
const firebase_functions_1 = require("./firebase-functions");
const util_1 = require("./util");
/**
 * Recursively returns a document referred by join view. Returns latest document
 * snapshot if it is the last document in the whole chain.
 *
 * @param app Firebase app.
 * @param refChain Chain of reference to the document.
 * @param snapshot Latest document snapshot in the chain.
 * @returns Document snapshot of first reference in current chain.
 */
async function getRefDocFromRefSpecChainRecursive(app, refChain, snapshot) {
    var _a;
    const [currentRefSpec, ...nextRefChain] = refChain;
    if (currentRefSpec === undefined) {
        return snapshot;
    }
    const refId = (_a = snapshot.data) === null || _a === void 0 ? void 0 : _a[currentRefSpec.fieldName];
    if (typeof refId !== 'string') {
        throw Error(`Invalid Type: ${JSON.stringify({
            data: snapshot.data,
            fieldName: currentRefSpec.fieldName,
        })}`);
    }
    const currentRefDoc = await (0, firebase_admin_1.getDoc)(app, currentRefSpec.collectionName, refId);
    const refDoc = getRefDocFromRefSpecChainRecursive(app, nextRefChain, currentRefDoc);
    return refDoc;
}
/**
 * Get a document referenced by a join view.
 *
 * @param app Firebase app.
 * @param joinSpec Specification of the join view.
 * @param data Data of source document of the join view.
 * @returns Document referenced.
 */
async function getRefDocFromRefSpecs(app, joinSpec, data) {
    const { firstRef, refChain } = joinSpec;
    const refId = data[firstRef.fieldName];
    if (typeof refId !== 'string') {
        throw Error(`Invalid Type: ${JSON.stringify({ data, firstRef })}`);
    }
    const firstRefDoc = await (0, firebase_admin_1.getDoc)(app, firstRef.collectionName, refId);
    const refDoc = getRefDocFromRefSpecChainRecursive(app, refChain, firstRefDoc);
    return refDoc;
}
/**
 * Prefix a join name to a field name.
 *
 * @param joinName Name of the join view.
 * @param fieldName Field name to prefix.
 * @returns Field name prefixed by join name.
 */
function prefixJoinName(joinName, fieldName) {
    const prefixedFieldName = `${joinName}_${fieldName}`;
    return prefixedFieldName;
}
/**
 * Make refId field name of a join view.
 *
 * @param joinName Name of the join view.
 * @returns Name of refId field of the view.
 */
function makeRefIdFieldName(joinName) {
    return prefixJoinName(joinName, 'id');
}
/**
 * Creates document data with fields name prefixed by join name.
 *
 * @param docData Document data to be process.
 * @param joinName Name of the join view
 * @returns Document data with prefixed fields name.
 */
function prefixJoinNameOnDocData(docData, joinName) {
    return (0, lodash_1.mapKeys)(docData, (_, fieldName) => prefixJoinName(joinName, fieldName));
}
/**
 * Materialize join view data from a specification.
 *
 * @param app Firebase app.
 * @param srcDocData Source document data of the join view.
 * @param spec Materialization specification.
 * @returns Materialized join view data.
 */
async function materializeJoinData(app, srcDocData, joinName, spec) {
    const refDoc = await getRefDocFromRefSpecs(app, spec, srcDocData);
    const selectedFieldDocData = (0, lodash_1.pick)(refDoc.data, spec.selectedFieldNames);
    const compactDocData = (0, util_1.compactObject)(selectedFieldDocData);
    const prefixedData = prefixJoinNameOnDocData(compactDocData, joinName);
    const refIdFieldName = makeRefIdFieldName(joinName);
    const docDataWithRefId = Object.assign(Object.assign({}, prefixedData), { [refIdFieldName]: refDoc.id });
    return docDataWithRefId;
}
/**
 * Maps array of join view specification into array of materialized data.
 *
 * @param app Firebase app.
 * @param srcDocData Source document data of the join view.
 * @param specs Array of join view specification.
 * @returns Array of materialized datas.
 */
async function materializeJoinViewData(app, srcDocData, specs) {
    const docDataPromises = Object.entries(specs).map(([joinName, spec]) => materializeJoinData(app, srcDocData, joinName, spec));
    const docDataArray = await Promise.all(docDataPromises);
    const docData = (0, util_1.mergeObjectArray)(docDataArray);
    return docData;
}
exports.materializeJoinViewData = materializeJoinViewData;
/**
 * Get collection name of the join view, which is the latest collection in the
 * chain.
 *
 * @param spec Specification of the view.
 * @returns Name of the view's reference collection.
 */
function makeJoinRefCollectionName({ refChain, firstRef }) {
    var _a, _b;
    return ((_b = (_a = refChain[refChain.length - 1]) === null || _a === void 0 ? void 0 : _a.collectionName) !== null && _b !== void 0 ? _b : firstRef.collectionName);
}
/**
 * Make a trigger that runs on update of the source document referenced by the
 * join view. The trigger will update all document with join view(s) referencing
 * to the source document.
 *
 * @param app Firebase app.
 * @param collectionName Name of the view's collection.
 * @param viewName Name of the view.
 * @param spec Specification of the join view.
 * @returns Trigger that runs on source document update.
 */
function makeOnJoinRefDocUpdatedTrigger(app, collectionName, viewName, joinName, spec) {
    const refCollectionName = makeJoinRefCollectionName(spec);
    return (0, firebase_functions_1.makeOnUpdateTrigger)(refCollectionName, async (refDoc) => {
        const allDocDataUpdate = (0, util_1.getDocDataChange)(refDoc.data);
        const docDataUpdate = (0, lodash_1.pick)(allDocDataUpdate, spec.selectedFieldNames);
        if ((0, lodash_1.isEmpty)(docDataUpdate)) {
            return;
        }
        const prefixedDocDataUpdate = prefixJoinNameOnDocData(docDataUpdate, joinName);
        const refIdFieldName = makeRefIdFieldName(joinName);
        const viewCollectionName = (0, util_1.getViewCollectionName)(collectionName, viewName);
        const referrerViews = await (0, firebase_admin_1.getCollection)(app, viewCollectionName, (collection) => collection.where(refIdFieldName, '==', refDoc.id));
        const referrerViewsUpdates = referrerViews.docs.map((doc) => (0, firebase_admin_1.updateDoc)(app, viewCollectionName, doc.id, prefixedDocDataUpdate));
        const promisesResult = await Promise.allSettled(referrerViewsUpdates);
        (0, util_1.throwRejectedPromises)(promisesResult);
    });
}
/**
 * Create triggers that run on update of documents that referenecd by a
 * collection's join views.
 *
 * @param app Firebase app.
 * @param collectionName Name of the collection.
 * @param viewName Name of the view.
 * @param joinSpecs Specifications of the collection's join views.
 * @returns Dictionary of triggers that run on join view's referenced document
 * updated.
 */
function onJoinRefDocUpdated(app, collectionName, viewName, joinSpecs) {
    return (0, lodash_1.mapValues)(joinSpecs, (joinSpec, joinName) => makeOnJoinRefDocUpdatedTrigger(app, collectionName, viewName, joinName, joinSpec));
}
exports.onJoinRefDocUpdated = onJoinRefDocUpdated;
//# sourceMappingURL=view-join.js.map